// Copyright (c) Project Jupyter Contributors
// Distributed under the terms of the Modified BSD License.
import midi from 'webmidi';
import { Signal } from '@phosphor/signaling';
import { Disposable } from './disposable';
/**
 * A button
 */
export class Button extends Disposable {
    /**
     * @param control - the note number corresponding to the button
     * @param light - true when there is an indicator light for the button that
     * should reflect the toggle state.
     */
    constructor(control, { mode = 'momentary', light = true } = {}) {
        super();
        this._click = new Signal(this);
        this._stateChanged = new Signal(this);
        this._toggled = false;
        this._control = control;
        this._mode = mode;
        this._light = light;
        const input = midi.inputs.find(x => x.manufacturer === "Behringer" && x.name.startsWith("X-TOUCH MINI"));
        if (!input) {
            throw new Error("Could not find Behringer X-TOUCH MINI");
        }
        input.addListener('noteon', 1, e => {
            if (this.mode === 'momentary' && e.note.number === this._control) {
                this.toggled = !this._toggled;
                this.refresh();
            }
        });
        input.addListener('noteoff', 1, e => {
            if (e.note.number === this._control) {
                this._click.emit(undefined);
                this.toggled = !this._toggled;
                this.refresh();
            }
        });
    }
    get toggled() {
        return this._toggled;
    }
    set toggled(newValue) {
        const oldValue = this._toggled;
        if (oldValue !== newValue) {
            this._toggled = newValue;
            this.refresh();
            this._stateChanged.emit({ name: 'toggled', oldValue, newValue });
        }
    }
    /**
     * The toggle mode for the button.
     *
     * The mode can be:
     * * 'momentary' - the button state is only changed momentarily while the button is held down
     * * 'toggle' - each full click of the button toggles the state
     */
    get mode() {
        return this._mode;
    }
    set mode(newValue) {
        const oldValue = this._mode;
        if (oldValue !== newValue) {
            this._mode = newValue;
            this.refresh();
            this._stateChanged.emit({ name: 'mode', oldValue, newValue });
        }
    }
    /**
     * Set the button light, if available.
     *
     * Possible values are:
     * * 0: off
     * * 1: blinking
     * * 127 (0x7F): on
     */
    setButtonLight(value) {
        const output = midi.outputs.find(x => x.manufacturer === "Behringer" && x.name.startsWith("X-TOUCH MINI"));
        if (!output) {
            throw new Error("Could not find Behringer X-TOUCH MINI");
        }
        if (this._light) {
            output.playNote(this._control, 1, {
                velocity: value,
                rawVelocity: true
            });
        }
    }
    /**
     * Refresh the indicated button state.
     */
    refresh() {
        this.setButtonLight(this._toggled ? 0x7f : 0);
    }
    /**
     * A signal fired when a button is clicked.
     */
    get click() {
        return this._click;
    }
    /**
     * A signal fired when the widget state changes.
     */
    get stateChanged() {
        return this._stateChanged;
    }
}
