// Copyright (c) Project Jupyter Contributors
// Distributed under the terms of the Modified BSD License.
import midi from 'webmidi';
import { clamp } from './utils';
import { Signal } from '@phosphor/signaling';
import { Disposable } from './disposable';
export class Fader extends Disposable {
    /**
     * control is the midi pitchblend control number.
     */
    constructor(control, { min = 0, max = 127, value = 0, motorized = false } = {}) {
        super();
        this._stateChanged = new Signal(this);
        this._control = control;
        this._motorized = motorized;
        this._min = min;
        this._max = max;
        // TODO: provide a 'pickup' fader mode?
        const input = midi.inputs.find(x => x.manufacturer === "Behringer" && x.name.startsWith("X-TOUCH MINI"));
        if (!input) {
            throw new Error("Could not find Behringer X-TOUCH MINI");
        }
        input.addListener('pitchbend', this._control, e => {
            // for the xtouch mini, we only have the 7 msb of the pitchblend number,
            // so we only use e.data[2]
            this.value = Math.round((this._max - this._min) * (e.data[2] / 127)) + this._min;
        });
        this.value = value;
    }
    refresh() {
        if (this._motorized) {
            // TODO: if this value change came from the controller fader, we don't need to send it back?
            const faderValue = ((this._value - this._min) / (this._max - this._min)) * 2 - 1;
            const output = midi.outputs.find(x => x.manufacturer === "Behringer" && x.name.startsWith("X-TOUCH MINI"));
            if (!output) {
                throw new Error("Could not find Behringer X-TOUCH MINI");
            }
            output.sendPitchBend(faderValue, this._control);
        }
    }
    /**
     * value goes from min to max, inclusive.
     */
    get value() {
        return this._value;
    }
    set value(value) {
        const newValue = clamp(value, this._min, this._max);
        const oldValue = this._value;
        if (oldValue !== newValue) {
            this._value = newValue;
            this.refresh();
            this._stateChanged.emit({
                name: 'value',
                oldValue,
                newValue
            });
        }
    }
    /**
     * Setting the min may bump the value and max if necessary.
     */
    get min() {
        return this._min;
    }
    set min(newValue) {
        const oldValue = this._min;
        if (oldValue !== newValue) {
            if (newValue > this.value) {
                this.value = newValue;
            }
            if (newValue > this.max) {
                this.max = newValue;
            }
            this._min = newValue;
            this.refresh();
            this._stateChanged.emit({
                name: 'min',
                oldValue,
                newValue
            });
        }
    }
    /**
     * Setting the max may bump the value and min if necessary.
     */
    get max() {
        return this._max;
    }
    set max(newValue) {
        const oldValue = this._max;
        if (oldValue !== newValue) {
            if (newValue < this.value) {
                this.value = newValue;
            }
            if (newValue < this.min) {
                this.min = newValue;
            }
            this._max = newValue;
            this.refresh();
            this._stateChanged.emit({
                name: 'max',
                oldValue,
                newValue
            });
        }
    }
    /**
     * A signal fired when the widget state changes.
     */
    get stateChanged() {
        return this._stateChanged;
    }
}
